@@ -77,7 +77,7 @@ static void gmpv_main_window_get_property(	GObject *object,
 						guint property_id,
 						GValue *value,
 						GParamSpec *pspec );
-static void size_allocate_handler(	GtkWidget *widget,
+static void resize_video_area_finalize(	GtkWidget *widget,
 					GdkRectangle *allocation,
 					gpointer data );
 static gboolean resize_to_target(gpointer data);
@@ -94,6 +94,10 @@ static void gmpv_main_window_constructed(GObject *object)
 	gtk_widget_hide(self->playlist);
 	gtk_widget_set_no_show_all(self->playlist, TRUE);
 
+	gtk_widget_show_all(self->control_box);
+	gtk_widget_hide(self->control_box);
+	gtk_widget_set_no_show_all(self->control_box, TRUE);
+
 	gtk_paned_pack1(	GTK_PANED(self->vid_area_paned),
 				self->vid_area,
 				TRUE,
@@ -141,7 +145,7 @@ static void gmpv_main_window_get_property(	GObject *object,
 	}
 }
 
-static void size_allocate_handler(	GtkWidget *widget,
+static void resize_video_area_finalize(	GtkWidget *widget,
 					GdkRectangle *allocation,
 					gpointer data )
 {
@@ -155,7 +159,7 @@ static void size_allocate_handler(	GtkWidget *widget,
 	gint target_height = wnd->resize_target[1];
 
 	g_signal_handlers_disconnect_by_func
-		(widget, size_allocate_handler, data);
+		(widget, resize_video_area_finalize, data);
 
 	/* Adjust resize offset */
 	if((width != target_width || height != target_height)
@@ -180,6 +184,12 @@ static gboolean resize_to_target(gpointer data)
 				target_width+wnd->width_offset,
 				target_height+wnd->height_offset );
 
+	/* Prevent graphical glitches that appear when calling
+	 * gmpv_main_window_resize_video_area() with the current size as the
+	 * target size.
+	 */
+	gmpv_playlist_widget_queue_draw(GMPV_PLAYLIST_WIDGET(wnd->playlist));
+
 	return FALSE;
 }
 
@@ -381,16 +391,19 @@ void gmpv_main_window_save_state(GmpvMainWindow *wnd)
 	gint height;
 	gint handle_pos;
 	gdouble volume;
+	gboolean controls_visible;
 
 	settings = g_settings_new(CONFIG_WIN_STATE);
 	handle_pos = gtk_paned_get_position(GTK_PANED(wnd->vid_area_paned));
 	volume = gmpv_control_box_get_volume(GMPV_CONTROL_BOX(wnd->control_box));
+	controls_visible = gtk_widget_get_visible(wnd->control_box);
 
 	gtk_window_get_size(GTK_WINDOW(wnd), &width, &height);
 
 	g_settings_set_int(settings, "width", width);
 	g_settings_set_int(settings, "height", height);
 	g_settings_set_double(settings, "volume", volume);
+	g_settings_set_boolean(settings, "show-controls", controls_visible);
 	g_settings_set_boolean(settings, "show-playlist", wnd->playlist_visible);
 
 	if(gmpv_main_window_get_playlist_visible(wnd))
@@ -417,17 +430,21 @@ void gmpv_main_window_load_state(GmpvMainWindow *wnd)
 		gint width = g_settings_get_int(settings, "width");
 		gint height = g_settings_get_int(settings, "height");
 		gint handle_pos;
+		gboolean controls_visible;
 		gdouble volume;
 
 		wnd->playlist_width
 			= g_settings_get_int(settings, "playlist-width");
 		wnd->playlist_visible
 			= g_settings_get_boolean(settings, "show-playlist");
+		controls_visible
+			= g_settings_get_boolean(settings, "show-controls");
 		volume = g_settings_get_double(settings, "volume");
 		handle_pos = width-(wnd->playlist_visible?wnd->playlist_width:0);
 
 		gmpv_control_box_set_volume
 			(GMPV_CONTROL_BOX(wnd->control_box), volume);
+		gtk_widget_set_visible(wnd->control_box, controls_visible);
 		gtk_widget_set_visible(wnd->playlist, wnd->playlist_visible);
 		gtk_window_resize(GTK_WINDOW(wnd), width, height);
 		gtk_paned_set_position
@@ -443,9 +460,9 @@ void gmpv_main_window_load_state(GmpvMainWindow *wnd)
 }
 
 void gmpv_main_window_update_track_list(	GmpvMainWindow *wnd,
-					const GSList *audio_list,
-					const GSList *video_list,
-					const GSList *sub_list )
+						const GSList *audio_list,
+						const GSList *video_list,
+						const GSList *sub_list )
 {
 	if(gmpv_main_window_get_csd_enabled(wnd))
 	{
@@ -476,12 +493,12 @@ void gmpv_main_window_update_track_list(	GmpvMainWindow *wnd,
 }
 
 void gmpv_main_window_resize_video_area(	GmpvMainWindow *wnd,
-					gint width,
-					gint height )
+						gint width,
+						gint height )
 {
 	g_signal_connect(	wnd->vid_area,
 				"size-allocate",
-				G_CALLBACK(size_allocate_handler),
+				G_CALLBACK(resize_video_area_finalize),
 				wnd );
 
 	wnd->resize_target[0] = width;
@@ -489,10 +506,10 @@ void gmpv_main_window_resize_video_area(	GmpvMainWindow *wnd,
 	resize_to_target(wnd);
 
 	/* The size may not change, so this is needed to ensure that
-	 * size_allocate_handler() will be called so that the event handler will
-	 * be disconnected.
+	 * resize_video_area_finalize() will be called so that the event handler
+	 * will be disconnected.
 	 */
-#if (GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 20)
+#if GTK_CHECK_VERSION(3, 20, 0)
 	gtk_widget_queue_allocate(wnd->vid_area);
 #else
 	gtk_widget_queue_resize(wnd->vid_area);
@@ -515,7 +532,7 @@ void gmpv_main_window_enable_csd(GmpvMainWindow *wnd)
 	fullscreen_icon = g_themed_icon_new_with_default_fallbacks
 				("view-fullscreen-symbolic");
 	menu_icon = g_themed_icon_new_with_default_fallbacks
-				("view-list-symbolic");
+				("open-menu-symbolic");
 
 	wnd->playlist_width = PLAYLIST_DEFAULT_WIDTH;
 	wnd->open_hdr_btn = gtk_menu_button_new();
@@ -560,7 +577,7 @@ void gmpv_main_window_enable_csd(GmpvMainWindow *wnd)
 
 	gtk_actionable_set_action_name
 		(	GTK_ACTIONABLE(wnd->fullscreen_hdr_btn),
-			"app.fullscreen_toggle" );
+			"app.toggle-fullscreen" );
 
 	gtk_paned_set_position(	GTK_PANED(wnd->vid_area_paned),
 				MAIN_WINDOW_DEFAULT_WIDTH
@@ -577,7 +594,8 @@ gboolean gmpv_main_window_get_csd_enabled(GmpvMainWindow *wnd)
 		wnd->menu_hdr_btn;
 }
 
-void gmpv_main_window_set_playlist_visible(GmpvMainWindow *wnd, gboolean visible)
+void gmpv_main_window_set_playlist_visible(	GmpvMainWindow *wnd,
+						gboolean visible )
 {
 	if(visible != wnd->playlist_visible && !wnd->fullscreen)
 	{
@@ -596,6 +614,14 @@ void gmpv_main_window_set_playlist_visible(GmpvMainWindow *wnd, gboolean visible
 		{
 			gint new_pos = width-(maximized?wnd->playlist_width:0);
 
+			if(gtk_check_version(3, 20, 0))
+			{
+				/* Workaround for window sizing bug affecting
+				 * GTK+ < 3.20
+				 */
+				new_pos -= 52;
+			}
+
 			gtk_paned_set_position(	GTK_PANED(wnd->vid_area_paned),
 						new_pos );
 		}
@@ -604,32 +630,42 @@ void gmpv_main_window_set_playlist_visible(GmpvMainWindow *wnd, gboolean visible
 			wnd->playlist_width = width-handle_pos;
 		}
 
-
 		wnd->playlist_visible = visible;
 		gtk_widget_set_visible(wnd->playlist, visible);
 
 		if(!maximized)
 		{
-#if (GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 20)
-			gtk_window_resize(	GTK_WINDOW(wnd),
-						visible?
+			if(!gtk_check_version(3, 20, 0))
+			{
+				gint new_width;
+
+				new_width =	visible?
 						width+wnd->playlist_width:
-						handle_pos,
-						height );
-#else
-			/* Workaround for window sizing bug on Wayland affecting
-			 * GTK+ < 3.20
-			 */
-			gint wnd_offset;
+						handle_pos;
 
-			wnd_offset = gmpv_main_window_get_csd_enabled(wnd)?52:0;
+				gtk_window_resize(	GTK_WINDOW(wnd),
+							new_width,
+							height );
+			}
+			else
+			{
+				/* Workaround for window sizing bug affecting
+				 * GTK+ < 3.20
+				 */
+				gboolean csd;
+				gint wnd_offset;
+				gint new_width;
 
-			gtk_window_resize(	GTK_WINDOW(wnd),
-						visible?
+				csd = gmpv_main_window_get_csd_enabled(wnd);
+				wnd_offset = csd?52:0;
+				new_width =	visible?
 						width+wnd->playlist_width-wnd_offset:
-						handle_pos+wnd_offset,
-						height );
-#endif
+						handle_pos+wnd_offset;
+
+				gtk_window_resize(	GTK_WINDOW(wnd),
+							new_width,
+							height );
+			}
 		}
 
 		wnd->playlist_first_toggle = FALSE;
