@@ -49,6 +49,7 @@
 #include "gmpv_control_box.h"
 #include "gmpv_playlist_widget.h"
 
+static void *get_proc_address(void *fn_ctx, const gchar *name);
 static void gmpv_mpv_obj_set_inst_property(	GObject *object,
 						guint property_id,
 						const GValue *value,
@@ -67,6 +68,26 @@ static void load_input_conf(GmpvMpvObj *mpv, const gchar *input_conf);
 
 G_DEFINE_TYPE(GmpvMpvObj, gmpv_mpv_obj, G_TYPE_OBJECT)
 
+static void *get_proc_address(void *fn_ctx, const gchar *name)
+{
+	GdkDisplay *display = gdk_display_get_default();
+
+#ifdef GDK_WINDOWING_WAYLAND
+	if (GDK_IS_WAYLAND_DISPLAY(display))
+		return eglGetProcAddress(name);
+#endif
+#ifdef GDK_WINDOWING_X11
+	if (GDK_IS_X11_DISPLAY(display))
+		return	(void *)(intptr_t)
+			glXGetProcAddressARB((const GLubyte *)name);
+#endif
+#ifdef GDK_WINDOWING_WIN32
+	if (GDK_IS_WIN32_DISPLAY(display))
+		return wglGetProcAddress(name);
+#endif
+	g_assert_not_reached();
+}
+
 static void gmpv_mpv_obj_set_inst_property(	GObject *object,
 						guint property_id,
 						const GValue *value,
@@ -156,7 +177,7 @@ static gboolean mpv_event_handler(gpointer data)
 
 			g_signal_emit_by_name(	mpv,
 						"mpv-prop-change",
-						g_strdup(prop->name) );
+						prop->name );
 		}
 		else if(event->event_id == MPV_EVENT_IDLE)
 		{
@@ -203,6 +224,8 @@ static gboolean mpv_event_handler(gpointer data)
 				gmpv_mpv_obj_set_property_flag
 					(mpv, "pause", TRUE);
 				g_signal_emit_by_name(mpv, "mpv-error", msg);
+
+				g_free(msg);
 			}
 		}
 		else if(event->event_id == MPV_EVENT_VIDEO_RECONFIG)
@@ -273,43 +296,59 @@ static void mpv_obj_update_playlist(GmpvMpvObj *mpv)
 
 	for(i = 0; i < playlist_count; i++)
 	{
-		gint prop_count = mpv_playlist.u.list->values[i].u.list->num;
+		mpv_node_list *prop_list = mpv_playlist.u.list->values[i].u.list;
 		gchar *uri = NULL;
 		gchar *title = NULL;
 		gchar *name = NULL;
-		gchar *old_name = NULL;
-		gchar *old_uri = NULL;
 
-		/* The first entry must always exist */
-		uri =	mpv_playlist.u.list
-			->values[i].u.list
-			->values[0].u.string;
-
-		/* Try retrieving the title from mpv playlist */
-		if(prop_count >= 4)
+		for(gint j = 0; j < prop_list->num; j++)
 		{
-			title = mpv_playlist.u.list
-				->values[i].u.list
-				->values[3].u.string;
+			const gchar *key = prop_list->keys[j];
+			const mpv_node value = prop_list->values[j];
+
+			if(g_strcmp0(key, "filename") == 0)
+			{
+				g_assert(value.format == MPV_FORMAT_STRING);
+				uri = value.u.string;
+			}
+			else if(g_strcmp0(key, "title") == 0)
+			{
+				g_assert(value.format == MPV_FORMAT_STRING);
+				title = value.u.string;
+			}
 		}
 
 		name = title?g_strdup(title):get_name_from_path(uri);
 
+		/* Overwrite current entry if it doesn't match the new value */
 		if(!iter_end)
 		{
+			gchar *old_name = NULL;
+			gchar *old_uri = NULL;
+			gboolean name_update;
+			gboolean uri_update;
+
 			gtk_tree_model_get
 				(	GTK_TREE_MODEL(store), &iter,
 					PLAYLIST_NAME_COLUMN, &old_name,
 					PLAYLIST_URI_COLUMN, &old_uri, -1 );
 
-			if(g_strcmp0(name, old_name) != 0)
+			name_update = (g_strcmp0(name, old_name) != 0);
+			uri_update = (g_strcmp0(uri, old_uri) != 0);
+
+			/* Only set the name if either the title can be
+			 * retrieved or the name is unset. This preserves the
+			 * correct title if it becomes unavailable later such as
+			 * when restarting mpv.
+			 */
+			if(name_update && (!old_name || title || uri_update))
 			{
 				gtk_list_store_set
 					(	store, &iter,
 						PLAYLIST_NAME_COLUMN, name, -1 );
 			}
 
-			if(g_strcmp0(uri, old_uri) != 0)
+			if(uri_update)
 			{
 				gtk_list_store_set
 					(	store, &iter,
@@ -559,7 +598,6 @@ static void gmpv_mpv_obj_class_init(GmpvMpvObjClass* klass)
 			G_MAXINT64,
 			-1,
 			G_PARAM_READWRITE );
-
 	g_object_class_install_property(obj_class, PROP_WID, pspec);
 
 	pspec = g_param_spec_pointer
@@ -567,7 +605,6 @@ static void gmpv_mpv_obj_class_init(GmpvMpvObjClass* klass)
 			"Playlist",
 			"GmpvPlaylist to use for storage",
 			G_PARAM_READWRITE );
-
 	g_object_class_install_property(obj_class, PROP_PLAYLIST, pspec);
 
 	g_signal_new(	"mpv-init",
@@ -579,7 +616,6 @@ static void gmpv_mpv_obj_class_init(GmpvMpvObjClass* klass)
 			g_cclosure_marshal_VOID__VOID,
 			G_TYPE_NONE,
 			0 );
-
 	g_signal_new(	"mpv-error",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -590,7 +626,6 @@ static void gmpv_mpv_obj_class_init(GmpvMpvObjClass* klass)
 			G_TYPE_NONE,
 			1,
 			G_TYPE_STRING );
-
 	g_signal_new(	"mpv-playback-restart",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -600,7 +635,6 @@ static void gmpv_mpv_obj_class_init(GmpvMpvObjClass* klass)
 			g_cclosure_marshal_VOID__VOID,
 			G_TYPE_NONE,
 			0 );
-
 	g_signal_new(	"mpv-event",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -611,7 +645,6 @@ static void gmpv_mpv_obj_class_init(GmpvMpvObjClass* klass)
 			G_TYPE_NONE,
 			1,
 			G_TYPE_INT );
-
 	g_signal_new(	"mpv-prop-change",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -900,11 +933,10 @@ inline mpv_opengl_cb_context *gmpv_mpv_obj_get_opengl_cb_context(GmpvMpvObj *mpv
 void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 {
 	GSettings *main_settings = g_settings_new(CONFIG_ROOT);
-	GSettings *win_settings = g_settings_new(CONFIG_WIN_STATE);
-	gdouble volume = g_settings_get_double(win_settings, "volume")*100;
 	gchar *config_dir = get_config_dir_path();
 	gchar *mpvopt = NULL;
 	gchar *current_vo = NULL;
+	gchar *mpv_version = NULL;
 
 	const struct
 	{
@@ -921,6 +953,7 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 			{"window-scale", "1"},
 			{"pause", "no"},
 			{"ytdl", "yes"},
+			{"osd-bar", "no"},
 			{"input-cursor", "no"},
 			{"cursor-autohide", "no"},
 			{"softvol-max", "100"},
@@ -942,9 +975,6 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 					options[i].value );
 	}
 
-	g_debug("Setting volume to %f", volume);
-	mpv_set_option(mpv->mpv_ctx, "volume", MPV_FORMAT_DOUBLE, &volume);
-
 	if(g_settings_get_boolean(main_settings, "mpv-config-enable"))
 	{
 		gchar *mpv_conf
@@ -983,7 +1013,7 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 		const gchar *msg
 			= _("Failed to apply one or more MPV options.");
 
-		g_signal_emit_by_name(mpv, "mpv-error", g_strdup(msg));
+		g_signal_emit_by_name(mpv, "mpv-error", msg);
 	}
 
 	if(mpv->force_opengl)
@@ -1003,6 +1033,7 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 	mpv_observe_property(mpv->mpv_ctx, 0, "aid", MPV_FORMAT_INT64);
 	mpv_observe_property(mpv->mpv_ctx, 0, "chapters", MPV_FORMAT_INT64);
 	mpv_observe_property(mpv->mpv_ctx, 0, "core-idle", MPV_FORMAT_FLAG);
+	mpv_observe_property(mpv->mpv_ctx, 0, "fullscreen", MPV_FORMAT_FLAG);
 	mpv_observe_property(mpv->mpv_ctx, 0, "pause", MPV_FORMAT_FLAG);
 	mpv_observe_property(mpv->mpv_ctx, 0, "length", MPV_FORMAT_DOUBLE);
 	mpv_observe_property(mpv->mpv_ctx, 0, "media-title", MPV_FORMAT_STRING);
@@ -1012,8 +1043,12 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 	mpv_set_wakeup_callback(mpv->mpv_ctx, wakeup_callback, mpv);
 	mpv_check_error(mpv_initialize(mpv->mpv_ctx));
 
+
+	mpv_version = gmpv_mpv_obj_get_property_string(mpv, "mpv-version");
 	current_vo = gmpv_mpv_obj_get_property_string(mpv, "current-vo");
 
+	g_info("Using %s", mpv_version);
+
 	if(current_vo && !GDK_IS_X11_DISPLAY(gdk_display_get_default()))
 	{
 		g_info(	"The chosen vo is %s but the display is not X11; "
@@ -1023,11 +1058,22 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 		mpv->force_opengl = TRUE;
 		mpv->state.paused = FALSE;
 
-		mpv_free(current_vo);
 		gmpv_mpv_obj_reset(mpv);
 	}
 	else
 	{
+		GSettings *win_settings;
+		gdouble volume;
+
+		win_settings = g_settings_new(CONFIG_WIN_STATE);
+		volume = g_settings_get_double(win_settings, "volume")*100;
+
+		g_debug("Setting volume to %f", volume);
+		mpv_set_property(	mpv->mpv_ctx,
+					"volume",
+					MPV_FORMAT_DOUBLE,
+					&volume );
+
 		/* The vo should be opengl-cb if current_vo is NULL*/
 		if(!current_vo)
 		{
@@ -1037,16 +1083,41 @@ void gmpv_mpv_obj_initialize(GmpvMpvObj *mpv)
 		}
 
 		gmpv_mpv_opt_handle_msg_level(mpv);
+		gmpv_mpv_opt_handle_fs(mpv);
 
 		mpv->force_opengl = FALSE;
 		mpv->state.ready = TRUE;
 		g_signal_emit_by_name(mpv, "mpv-init");
+
+		g_clear_object(&win_settings);
 	}
 
 	g_clear_object(&main_settings);
-	g_clear_object(&win_settings);
 	g_free(config_dir);
 	g_free(mpvopt);
+	mpv_free(current_vo);
+	mpv_free(mpv_version);
+}
+
+void gmpv_mpv_obj_init_gl(GmpvMpvObj *mpv)
+{
+	mpv_opengl_cb_context *opengl_ctx;
+	gint rc;
+
+	opengl_ctx = gmpv_mpv_obj_get_opengl_cb_context(mpv);
+	rc = mpv_opengl_cb_init_gl(	opengl_ctx,
+					NULL,
+					get_proc_address,
+					NULL );
+
+	if(rc >= 0)
+	{
+		g_debug("Initialized opengl-cb");
+	}
+	else
+	{
+		g_critical("Failed to initialize opengl-cb");
+	}
 }
 
 void gmpv_mpv_obj_reset(GmpvMpvObj *mpv)
@@ -1249,15 +1320,17 @@ void gmpv_mpv_obj_load(	GmpvMpvObj *mpv,
 	}
 }
 
+void gmpv_mpv_obj_free(gpointer data)
+{
+	mpv_free(data);
+}
+
 void gmpv_mpv_obj_load_list(	GmpvMpvObj *mpv,
 				const gchar **uri_list,
 				gboolean append,
 				gboolean update )
 {
-	static const char *const sub_exts[]
-		= {	"utf", "utf8", "utf-8", "idx", "sub", "srt", "smi",
-			"rt", "txt", "ssa", "aqt", "jss", "js", "ass", "mks",
-			"vtt", "sup", NULL };
+	static const char *const sub_exts[] = SUBTITLE_EXTS;
 
 	for(gint i = 0; uri_list[i]; i++)
 	{
